@@ -4,10 +4,15 @@ std <- function(X) {
     if (class(tmp)[1] == "try-error") stop("X must be a matrix or able to be coerced to a matrix")
   }
   STD <- .Call("standardize", X)
-  val <- STD[[1]]
-  attr(val, "center") <- STD[[2]]
+  dimnames(STD[[1]]) <- dimnames(X)
+  ns <- which(STD[[3]] > 1e-6)
+  if (length(ns) == ncol(X)) {
+    val <- STD[[1]]
+  } else {
+    val <- STD[[1]][, ns, drop=FALSE]
+  }
   attr(val, "center") <- STD[[2]]
   attr(val, "scale") <- STD[[3]]
-  dimnames(val) <- dimnames(X)
+  attr(val, "nonsingular") <- ns
   val
 }
